package javax.print.attribute.standard;

import java.util.NoSuchElementException;
import java.util.AbstractSet;
import java.util.Set;
import javax.print.attribute.Attribute;
import java.util.Iterator;
import java.util.Map;
import javax.print.attribute.PrintServiceAttribute;
import java.util.HashMap;

public final class PrinterStateReasons extends HashMap<PrinterStateReason, Severity> implements PrintServiceAttribute
{
    private static final long serialVersionUID = -3731791085163619457L;
    
    public PrinterStateReasons() {
    }
    
    public PrinterStateReasons(int initialCapacity) {
        super(initialCapacity);
    }
    
    public PrinterStateReasons(int initialCapacity, float loadFactor) {
        super(initialCapacity, loadFactor);
    }
    
    public PrinterStateReasons(Map<PrinterStateReason, Severity> map) {
        this();
        for (Map.Entry entry : map.entrySet()) {
            this.put((PrinterStateReason)entry.getKey(), (Severity)entry.getValue());
        }
    }
    
    @Override
    public Severity put(PrinterStateReason key, Severity value) {
        if (key == null) {
            throw new NullPointerException("reason is null");
        }
        if (value == null) {
            throw new NullPointerException("severity is null");
        }
        return (Severity)super.put(key, value);
    }
    
    @Override
    public final Class<? extends Attribute> getCategory() {
        return PrinterStateReasons.class;
    }
    
    @Override
    public final String getName() {
        return "printer-state-reasons";
    }
    
    public Set<PrinterStateReason> printerStateReasonSet(Severity severity) {
        if (severity == null) {
            throw new NullPointerException("severity is null");
        }
        return new PrinterStateReasons.PrinterStateReasonSet(severity, this.entrySet());
    }
    
    private class PrinterStateReasonSet extends AbstractSet<PrinterStateReason>
    {
        private Severity mySeverity;
        private Set myEntrySet;
        
        public PrinterStateReasonSet(Severity mySeverity, Set myEntrySet) {
            this.mySeverity = mySeverity;
            this.myEntrySet = myEntrySet;
        }
        
        @Override
        public int size() {
            int n = 0;
            for (Iterator iterator = this.iterator(); iterator.hasNext(); ++n) {
                iterator.next();
            }
            return n;
        }
        
        @Override
        public Iterator iterator() {
            return new PrinterStateReasons.PrinterStateReasonSetIterator(this.mySeverity, this.myEntrySet.iterator());
        }
    }
    
    private class PrinterStateReasonSetIterator implements Iterator
    {
        private Severity mySeverity;
        private Iterator myIterator;
        private Map.Entry myEntry;
        
        public PrinterStateReasonSetIterator(Severity mySeverity, Iterator myIterator) {
            this.mySeverity = mySeverity;
            this.myIterator = myIterator;
            this.goToNext();
        }
        
        private void goToNext() {
            this.myEntry = null;
            while (this.myEntry == null && this.myIterator.hasNext()) {
                this.myEntry = (Map.Entry)this.myIterator.next();
                if ((Severity)this.myEntry.getValue() == this.mySeverity) {
                    continue;
                }
                this.myEntry = null;
            }
        }
        
        @Override
        public boolean hasNext() {
            return this.myEntry != null;
        }
        
        @Override
        public Object next() {
            if (this.myEntry == null) {
                throw new NoSuchElementException();
            }
            Object key = this.myEntry.getKey();
            this.goToNext();
            return key;
        }
        
        @Override
        public void remove() {
            throw new UnsupportedOperationException();
        }
    }
}
